<?php

if (!defined('PLX_ROOT')) { exit; }

/**
 * Permet à un visiteur de tester les différents styles avec un fichier infos.xml
 * présents dans le dossier de thèmes.
 *
 * La boîte de seélection de thèmes s'affiche en base et à droite du site.
 * Il n'y a aucun hook à ajouter aux thèmes. Activez simplement ce plugin pour vos essais.
 *
 * @author	J.P. Pourrez aka Bazooka07
 * @update  2021-06-02
 * @update	2019-11-27
 * @date	2018-12-29
 * */
class kzSkinSelect extends plxPlugin {

	const TOKEN = __CLASS__ .'-Token';
	const FIELD_NAME = 'kz-token';
	const HOOKS = array(
		'plxMotorDemarrageBegin',
		'plxMotorDemarrageEnd',
		'ThemeEndHead',
		'ThemeEndBody',
	);
	const BEGIN_CODE = '<?php # ' . __CLASS__ . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';

	private $__currentTheme = false;

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		if(!defined('PLX_ADMIN')) {
			foreach(self::HOOKS as $hook) {
				$this->addHook($hook, $hook);
			}
		}
	}

	private function _title_theme($filename) {
		if(function_exists('simplexml_load_file')) {
			$infos = simplexml_load_file($filename);
			return trim($infos->title->__toString());
		} else {
			$parser = xml_parser_create(PLX_CHARSET);
			xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
			xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 0);
			if(xml_parse_into_struct($parser, file_get_contents($filename), $values, $tags) == 1) {
				xml_parser_free($parser);
				if(!empty($tags['title'])) {
					$k = $tags['title'][0];
					return $values[$k]['value'];
				}
			}
		}
		return 'No-title';
	}

	/**
	 * Traitement du hook plxMotorDemarrageBegin.
	 * */
	public function getThemes($themesRoot, $currentTheme) {
		$themes = array();
		// On collecte les thèmes disponibles sur le site
        $pattern = PLX_ROOT . $themesRoot . '*/infos.xml';
		foreach(glob($pattern) as $filename) {

			$root = preg_replace('@/infos.xml$@', '', $filename);
            $folder = basename($root);
			$caption = self::_title_theme($filename);
			if(strtolower($caption) != strtolower($folder)) {
				$caption .= " ($folder)";
			}
            // Tague les thèmes sans aperçu
            $mark = ' *';
            foreach(array('jpg', 'jpeg', 'png', 'gif') as $ext) {
				if(file_exists($root."/preview.$ext")) {
					$mark = '';
					break;
				}
			}
            $themes[$folder] = $caption.$mark;
		}

		if(count($themes) > 1) {
			// Plusieurs thèmes sont disponibles
			asort($themes);
			$this->__themes = $themes;
			$this->__currentTheme = $currentTheme;

			$options = array(
				'options' => array('regexp' => '@^[\da-z]{40}@i')
			);
			if(filter_has_var(INPUT_POST, __CLASS__)) {
				if(isset($_SESSION[self::TOKEN])) {
					if(
						empty($_POST[self::FIELD_NAME]) or
						filter_input(INPUT_POST, self::FIELD_NAME, FILTER_VALIDATE_REGEXP, $options) === false
					) {
						return false;
					}

					$key = $_POST[self::FIELD_NAME];
					if(isset($_SESSION[self::TOKEN][$key]) and intval($_SESSION[self::TOKEN][$key]) < time()) {
						unset($_SESSION[self::TOKEN]);
						die('Security error : invalid or expired token');
					}
					unset($_SESSION[self::TOKEN][$key]);
					// On change de thème
					$value = filter_input(INPUT_POST, __CLASS__, FILTER_SANITIZE_STRING);
					if(!empty($value) and array_key_exists($value, $themes)) {
						$this->__currentTheme = $value;
						$_SESSION[__CLASS__] = $value;
						return $value;
					}
				}
			} elseif(!empty($_SESSION[__CLASS__])) {
				// On récupère le précèdent choix du visiteur et on l'active s'il existe.
				$value = $_SESSION[__CLASS__];
				if(array_key_exists($value, $themes)) {
					$this->__currentTheme = $value;
					return $value;
				}
			}
		}

		return false;
	}

	/* ============== Hooks ========================= */

	/**
	 * Analyse de la situation.
	 * valide le choix d'un thème par le visiteur.
	 * En mode article, interdit les commentaires dans ce cas précis.
	 *
	 * Dans le cas contraire, reprend le choix précèdent du visiteur stocké dans une variable de session.
	 * */
	public function plxMotorDemarrageBegin() {
		echo self::BEGIN_CODE;
?>
$value = $this->plxPlugins->aPlugins['<?= __CLASS__ ?>']->getThemes(
	$this->aConf['racine_themes'],
	$this->style
);

if(!empty($value)) {
	$this->style = $value;
}

if($this->mode == 'article' and filter_has_var(INPUT_POST, '<?= __CLASS__ ?>')) {
	$<?= __CLASS__ ?>Coms = $this->aConf['allow_com'];
	$this->aConf['allow_com'] = false;
}

# PluXml doit poursuivre son traitement
return false;
<?php
		echo self::END_CODE;
	}

	/**
	 * En mode article, restitue l'autorisation des commentaires.
	 * */
	public function plxMotorDemarrageEnd() {
		if(empty($this->__currentTheme)) {
			# Un seul thème disponible. Retour au fonctionnment standard
			return;
		}

		echo self::BEGIN_CODE;
?>
if(isset($<?= __CLASS__ ?>Coms)) {
	# On restaure l'autorisation des commentaires
	$this->aConf['allow_com'] = $<?= __CLASS__ ?>Coms;
}
<?php
		echo self::END_CODE;
	}

	/**
	 * Quelques rèegle de style CSS
	 * */
	public function ThemeEndHead() {
		if($this->__currentTheme === false) { return; }
?>
		<style type="text/css">
			#kzSkinSelect-wrapper {
				display: block;
				position: fixed;
				bottom: 1vh;
				right: 2rem;
				padding: 0.25rem;
				background-color: #bfb658;
				color: #000;
				font-family: 'Noto Sans', Arial, Sans-Serif;
				font-size: 12pt;
				border-radius: 0.3rem;
				-webkit-appearance: initial;
				z-index: 9999;
			}
			
			#kzSkinSelect-wrapper summary {
				display: list-item;
			}
			
			#kzSkinSelect-wrapper select {
				margin: 0;
				background-color: #fff;
			}
		</style>
<?php		
	}
	 
	/**
	 * Ajoute un formulaire en bas de page pour sélectionner un thème.
	 * */
	public function ThemeEndBody() {
		if($this->__currentTheme === false) { return; }

		$token = sha1(mt_rand(0, 1000000));
		$_SESSION[self::TOKEN][$token] = time() + 3600; // Date limite pour le token
?>
			<details id="<?= __CLASS__ ?>-wrapper">
				<summary>Style</summary>
				<form method="post">
					<input name="<?= self::FIELD_NAME; ?>" value="<?= $token; ?>" type="hidden" />
<?php plxUtils::printSelect(__CLASS__, $this->__themes, $this->__currentTheme); ?>
				</form>
			</details>
			<script type="text/javascript">
				(function() {
					'use strict';
					const details = document.getElementById('<?= __CLASS__ ?>-wrapper');
					const select = document.getElementById('id_<?= __CLASS__ ?>');
					
					if(details == null || select == null) { return; }
					
					select.autofocus = true;
					select.onchange = function(event) {
						event.preventDefault();
						if(select.value.trim() != '') {
							select.form.submit();
						}
					}
				})();
			</script>
<?php
	}

}
